import { ReplaySubject } from 'rxjs';
import { env } from '../../misc/env';
import { log } from '../../misc/util';
var Remote = (function () {
    function Remote(id, pc, send, remotes, timeout) {
        var _this = this;
        this.finalMessageReceived = false;
        this.finalMessageSent = false;
        this.id = id;
        this.pc = pc;
        this.send = send;
        this._onError = function () { };
        this.candidates = new ReplaySubject();
        this.remotes = remotes;
        this.isSDPSent = false;
        this.remotes.set(id, this);
        this.timer = setTimeout(function () { return _this._onError(new Error(timeout + "ms connection timeout")); }, timeout);
        pc.oniceconnectionstatechange = function () {
            log.webrtc('LOCAL ICE CONNECTION STATE', pc.iceConnectionState);
            if (pc.iceConnectionState === 'failed') {
                _this._onError(new Error('Ice Connection State is FAILED'));
            }
        };
        pc.onicecandidate = function (evt) {
            if (evt.candidate !== null) {
                log.webrtc('LOCAL ICE CANDIDATE', evt.candidate.candidate);
                _this.send({
                    candidate: {
                        candidate: evt.candidate.candidate,
                        sdpMid: evt.candidate.sdpMid,
                        sdpMLineIndex: evt.candidate.sdpMLineIndex,
                    },
                });
            }
            else {
                _this.pc.onicecandidate = function () { };
                if (_this.isSDPSent) {
                    _this.sendFinalMessage();
                }
            }
        };
    }
    Object.defineProperty(Remote.prototype, "onError", {
        get: function () {
            return this._onError;
        },
        set: function (handler) {
            var _this = this;
            this._onError = function (err) {
                log.webrtc('ERROR: ', err.message);
                _this.clean(err.message !== 'clean');
                handler(err);
            };
        },
        enumerable: true,
        configurable: true
    });
    Remote.prototype.sdpIsSent = function () {
        this.isSDPSent = true;
        if (this.pc.iceGatheringState === 'complete') {
            this.sendFinalMessage();
        }
    };
    Remote.prototype.clean = function (sendFinalMessage) {
        if (sendFinalMessage === void 0) { sendFinalMessage = true; }
        log.webrtc('CLEAN');
        this.pc.oniceconnectionstatechange = function () { };
        this.pc.onicecandidate = function () { };
        this.pc.ondatachannel = function () { };
        this._onError = function () { };
        this.candidates.complete();
        this.remotes.delete(this.id);
        clearTimeout(this.timer);
        this.pc.close();
        if (sendFinalMessage) {
            this.sendFinalMessage();
        }
    };
    Remote.prototype.dataChannelOpen = function (dc) {
        log.webrtc("DATA CHANNEL with " + this.id + " OPEN");
        if (this.finalMessageReceived && this.finalMessageSent) {
            this.remotes.delete(this.id);
        }
        this.pc.oniceconnectionstatechange = function () { };
        dc.onopen = function () { };
        this.pc.ondatachannel = function () { };
        this._onError = function () { };
        clearTimeout(this.timer);
    };
    Remote.prototype.handleMessage = function (msg) {
        var _this = this;
        if (msg.type) {
            switch (msg.type) {
                case 'offer':
                    log.webrtc('REMOTE OFFER', { offer: msg.offer });
                    this.pc
                        .setRemoteDescription({ type: 'offer', sdp: msg.offer })
                        .then(function () {
                        return _this.candidates.subscribe(function (ic) {
                            return _this.pc
                                .addIceCandidate(ic)
                                .catch(function (err) { return log.webrtc('Failed to addIceCandidate', err); });
                        });
                    })
                        .then(function () { return _this.pc.createAnswer(); })
                        .then(function (answer) { return _this.pc.setLocalDescription(answer); })
                        .then(function () {
                        var sdp = _this.pc.localDescription.sdp;
                        log.webrtc('Send LOCAL ANSWER', { answer: sdp });
                        _this.send({ answer: sdp });
                        _this.sdpIsSent();
                    })
                        .catch(function (err) { return _this._onError(err); });
                    break;
                case 'answer':
                    log.webrtc('REMOTE ANSWER is received', { answer: msg.answer });
                    this.pc
                        .setRemoteDescription({ type: 'answer', sdp: msg.answer })
                        .then(function () {
                        _this.candidates.subscribe(function (ic) {
                            return _this.pc
                                .addIceCandidate(ic)
                                .catch(function (err) { return log.webrtc(_this.id + ": Failed to add REMOTE Ice Candidate", err); });
                        });
                    })
                        .catch(function (err) { return _this._onError(err); });
                    break;
                case 'candidate':
                    log.webrtc('REMOTE ICE CANDIDATE is received', msg.candidate);
                    this.candidates.next(new env.RTCIceCandidate(msg.candidate));
                    break;
                default:
                    this._onError(new Error('Buffer Protocol unknown message from the remote peer'));
            }
        }
        else {
            log.webrtc('REMOTE FINAL MESSAGE received');
            this.finalMessageReceived = true;
            this.candidates.complete();
            if (this.finalMessageSent) {
                this.remotes.delete(this.id);
            }
        }
    };
    Remote.prototype.sendFinalMessage = function () {
        if (!this.finalMessageSent) {
            this.finalMessageSent = true;
            this.send({});
            if (this.finalMessageReceived) {
                this.remotes.delete(this.id);
            }
        }
    };
    return Remote;
}());
export { Remote };
//# sourceMappingURL=Remote.js.map